"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SparseMerkleTreeImpl = void 0;
const core_utils_1 = require("@eth-optimism/core-utils");
const assert = require("assert");
const AsyncLock = require("async-lock");
const domain = require("domain");
const db_interface_1 = require("../../types/db/db.interface");
const log = core_utils_1.getLogger('merkle-tree');
class SparseMerkleTreeImpl {
    constructor(db, height = 160, hashFunction = core_utils_1.keccak256) {
        this.db = db;
        this.height = height;
        this.treeLock = new AsyncLock({
            domainReentrant: true,
        });
        this.hashBuffer = Buffer.alloc(64);
        assert(height > 0, 'SMT height needs to be > 0');
        this.hashFunction = (buff) => Buffer.from(hashFunction(buff.toString('hex')), 'hex');
        this.nodeIDsToDelete = [];
    }
    static async create(db, rootHash, height = 160, hashFunction = core_utils_1.keccak256) {
        assert(!rootHash || rootHash.length === 32, 'Root hash must be 32 bytes');
        const tree = new SparseMerkleTreeImpl(db, height, hashFunction);
        await tree.init(rootHash);
        return tree;
    }
    async init(rootHash) {
        await this.populateZeroHashesAndRoot(rootHash);
    }
    getHeight() {
        return this.height;
    }
    async getRootHash() {
        const copy = Buffer.alloc(this.root.hash.length);
        log.debug(`Root Hash: [${this.root.hash.toString('hex')}], value: [${!!this.root.value ? this.root.value.toString('hex') : 'undefined'}]`);
        this.root.hash.copy(copy);
        return copy;
    }
    async purgeOldNodes() {
        const batch = this.nodeIDsToDelete.map((id) => {
            return {
                type: db_interface_1.DEL_BATCH_TYPE,
                key: id,
            };
        });
        await this.db.batch(batch);
        this.nodeIDsToDelete.length = 0;
    }
    async getLeaf(leafKey, rootHash) {
        log.debug(`Trying to get leaf [${leafKey.toString(10)}]`);
        return this.treeLock.acquire(SparseMerkleTreeImpl.lockKey, async () => {
            if (!!rootHash && !rootHash.equals(this.root.hash)) {
                log.debug(`Cannot get Leaf [${leafKey.toString(10)}] because root hash does not match.`);
                return undefined;
            }
            const nodesInPath = await this.getNodesInPath(leafKey);
            if (!nodesInPath || nodesInPath.length !== this.height) {
                log.debug(`Cannot get Leaf [${leafKey.toString(10)}] because nodes in path does not equal tree height. Expected [${this.height}], found [${nodesInPath.length}]`);
                return undefined;
            }
            const leaf = nodesInPath[nodesInPath.length - 1];
            if (!leaf.key.equals(leafKey)) {
                log.debug(`Cannot get Leaf because leaf key does not match. Path: [${leafKey.toString(10)}], leaf key: ${leaf.key.toString(10)}.`);
                return undefined;
            }
            log.debug(`Returning leaf value: [${leaf.value.toString()}].`);
            return leaf.value;
        });
    }
    async verifyAndStore(inclusionProof) {
        if (inclusionProof.siblings.length !== this.height - 1) {
            log.info(`verifyAndStore: inclusion proof siblings length [${inclusionProof.siblings.length}] != tree height -1 [${this.height - 1}]`);
            return false;
        }
        return this.treeLock.acquire(SparseMerkleTreeImpl.lockKey, async () => {
            const leafHash = this.hashFunction(inclusionProof.value);
            if (!!(await this.getNode(leafHash, inclusionProof.key))) {
                return true;
            }
            let child = this.createNode(leafHash, inclusionProof.value, inclusionProof.key);
            let siblingIndex = 0;
            let parent = child;
            const nodesToStore = [child];
            for (let parentDepth = this.height - 2; parentDepth >= 0; parentDepth--) {
                child = parent;
                const childDepth = parentDepth + 1;
                const childSiblingHash = inclusionProof.siblings[siblingIndex++];
                parent = this.calculateParentNode(child, childSiblingHash, inclusionProof.key, parentDepth);
                nodesToStore.push(parent);
                const siblingNode = await this.createProofSiblingNodeIfDoesntExist(childSiblingHash, inclusionProof.key, childDepth);
                if (!!siblingNode) {
                    nodesToStore.push(siblingNode);
                }
            }
            if (!parent.hash.equals(this.root.hash)) {
                log.info(`verifyAndStore: Parent hash [${parent.hash.toString('hex')}] does not equal root hash [${this.root.hash.toString('hex')}].`);
                return false;
            }
            const toDelete = await this.getNodesInPath(inclusionProof.key);
            await this.db.batch(this.getNodePutBatch(nodesToStore));
            this.root = parent;
            this.nodeIDsToDelete.push(...toDelete.map((n) => this.getNodeID(n)));
            return true;
        });
    }
    async update(leafKey, leafValue, purgeOldNodes = true, d) {
        return core_utils_1.runInDomain(d, async () => {
            return this.treeLock.acquire(SparseMerkleTreeImpl.lockKey, async () => {
                let nodesToUpdate = await this.getNodesInPath(leafKey);
                if (!nodesToUpdate) {
                    log.info(`Unable to update tree leaf ${leafKey.toString('hex')} to ${leafValue.toString('hex')} because there are no nodes in the leaf path.`);
                    return false;
                }
                else if (nodesToUpdate.length !== this.height) {
                    log.debug(`nodes to update length != height`);
                    if (!(await this.verifyAndStorePartiallyEmptyPath(leafKey, nodesToUpdate.length))) {
                        log.info(`Unable to update tree leaf ${leafKey.toString('hex')} to ${leafValue.toString('hex')} because unable to verify and store partially empty path.`);
                        return false;
                    }
                    nodesToUpdate = await this.getNodesInPath(leafKey);
                }
                const oldLeaf = nodesToUpdate[nodesToUpdate.length - 1];
                if (!!oldLeaf && !!oldLeaf.value && leafValue.equals(oldLeaf.value)) {
                    log.info(`Trying to update tree leaf ${leafKey.toString('hex')} to the same value it currently has ${leafValue.toString('hex')}. Returning.`);
                    return true;
                }
                const idsToDelete = [this.getNodeID(oldLeaf)];
                let updatedChild = {
                    key: leafKey,
                    value: leafValue,
                    hash: this.hashFunction(leafValue),
                };
                const nodesToSave = [updatedChild];
                let parentDepth = nodesToUpdate.length - 2;
                for (; parentDepth >= 0; parentDepth--) {
                    idsToDelete.push(this.getNodeID(nodesToUpdate[parentDepth]));
                    updatedChild = this.updateNode(nodesToUpdate[parentDepth], updatedChild, leafKey, parentDepth);
                    nodesToSave.push(updatedChild);
                }
                await this.db.batch(this.getNodePutBatch(nodesToSave));
                this.nodeIDsToDelete.push(...idsToDelete);
                if (purgeOldNodes) {
                    await this.purgeOldNodes();
                }
                this.root = updatedChild;
                log.debug(`Tree leaf 
          ${leafKey.toString('hex')} updated to value
          ${leafValue.toString('hex')} resulting in new root hash: 
          ${this.root.hash.toString('hex')}, value: 
          ${this.root.value.toString('hex')}.`);
                return true;
            });
        });
    }
    async batchUpdate(updates) {
        const d = domain.create();
        return core_utils_1.runInDomain(d, () => {
            return this.treeLock.acquire(SparseMerkleTreeImpl.lockKey, async () => {
                for (const update of updates) {
                    if (!(await this.verifyAndStore({
                        rootHash: this.root.hash,
                        key: update.key,
                        value: update.oldValue,
                        siblings: update.oldValueProofSiblings,
                    }))) {
                        return false;
                    }
                }
                const oldRoot = this.root;
                for (const update of updates) {
                    if (!(await this.update(update.key, update.newValue, false, d))) {
                        this.root = oldRoot;
                        return false;
                    }
                }
                return true;
            });
        });
    }
    async getMerkleProof(leafKey, leafValue) {
        const result = await this.treeLock.acquire(SparseMerkleTreeImpl.lockKey, async () => {
            if (!this.root || !this.root.hash) {
                return undefined;
            }
            let node = this.root;
            const siblings = [];
            for (let depth = 0; depth < this.height &&
                !!node &&
                !!node.value &&
                node.value.length === 64; depth++) {
                siblings.push(this.getChildSiblingHash(node, depth, leafKey));
                node = await this.getChild(node, depth, leafKey);
            }
            if (siblings.length !== this.height - 1) {
                log.info(`Getting merkle proof for leaf that has not been stored. Storing siblings and leaf and creating proof.`);
                return {
                    rootHash: undefined,
                    key: undefined,
                    value: undefined,
                    siblings: undefined,
                };
            }
            const leafHash = this.hashFunction(leafValue);
            if (!node.hash.equals(leafHash)) {
                log.info(`Node hash does not match leaf value hash for key [${leafKey.toNumber()}]. Passed leaf value: [${leafValue.toString()}], Passed leaf hash: ${leafHash.toString('hex')}, Node hash: ${node.hash.toString('hex')}`);
                return undefined;
            }
            return {
                rootHash: this.root.hash,
                key: leafKey,
                value: leafValue,
                siblings: siblings.reverse(),
            };
        });
        if (!result || !!result.rootHash) {
            return result;
        }
        if (leafValue.equals(SparseMerkleTreeImpl.emptyBuffer)) {
            if (await this.verifyAndStorePartiallyEmptyPath(leafKey)) {
                return this.getMerkleProof(leafKey, leafValue);
            }
        }
        return undefined;
    }
    async verifyAndStorePartiallyEmptyPath(leafKey, numExistingNodes) {
        if (numExistingNodes === undefined) {
            numExistingNodes = (await this.getNodesInPath(leafKey)).length;
        }
        const existingChildren = Math.max(numExistingNodes - 1, 0);
        const siblings = [];
        let node = this.root;
        for (let i = 0; i < this.height - 1; i++) {
            if (i > existingChildren ||
                (i === existingChildren && (!node.value || node.value.length !== 64))) {
                siblings.push(...this.zeroHashes.slice(i + 1));
                break;
            }
            siblings.push(this.getChildSiblingHash(node, i, leafKey));
            node = await this.getChild(node, i, leafKey);
        }
        return this.verifyAndStore({
            rootHash: this.root.hash,
            key: leafKey,
            value: SparseMerkleTreeImpl.emptyBuffer,
            siblings: siblings.reverse(),
        });
    }
    getChildSiblingHash(parent, parentDepth, leafKey) {
        const isLeft = this.isLeft(leafKey, parentDepth);
        return isLeft ? parent.value.subarray(32) : parent.value.subarray(0, 32);
    }
    async getChild(parent, parentDepth, leafKey) {
        const childIndex = this.isLeft(leafKey, parentDepth) ? 0 : 32;
        const childHash = parent.value.subarray(childIndex, childIndex + 32);
        return this.getNode(childHash, this.getNodeKey(leafKey, ++parentDepth));
    }
    async getNodesInPath(leafKey) {
        if (!this.root || !this.root.hash) {
            return [];
        }
        if (!this.root.value) {
            return [this.root];
        }
        let node = this.root;
        const nodesToUpdate = [node];
        let depth;
        for (depth = 0; depth < this.height - 1; depth++) {
            const childDepth = depth + 1;
            if (node.value.length === 64) {
                node = this.isLeft(leafKey, depth)
                    ? await this.getNode(node.value.subarray(0, 32), this.getNodeKey(leafKey, childDepth))
                    : await this.getNode(node.value.subarray(32), this.getNodeKey(leafKey, childDepth));
                if (node) {
                    nodesToUpdate.push(node);
                }
                else {
                    break;
                }
            }
            else {
                break;
            }
        }
        return nodesToUpdate;
    }
    updateNode(node, updatedChild, key, depth) {
        const value = Buffer.alloc(64, node.value);
        const isLeft = this.isLeft(key, depth);
        if (isLeft) {
            value.fill(updatedChild.hash, 0, 32);
        }
        else {
            value.fill(updatedChild.hash, 32);
        }
        return {
            key: node.key,
            value,
            hash: this.hashFunction(value),
        };
    }
    async createProofSiblingNodeIfDoesntExist(nodeHash, leafKey, depth) {
        const nodeKey = this.getNodeKey(leafKey, depth).xor(core_utils_1.ONE);
        const node = await this.getNode(nodeHash, nodeKey);
        if (!!node) {
            return undefined;
        }
        return this.createNode(nodeHash, SparseMerkleTreeImpl.siblingBuffer, nodeKey);
    }
    async getNode(nodeHash, nodeKey) {
        const value = await this.db.get(this.getNodeIDFromHashAndKey(nodeHash, nodeKey));
        if (!value) {
            return undefined;
        }
        return this.createNode(nodeHash, value, nodeKey);
    }
    calculateParentNode(node, siblingHash, leafKey, depth) {
        const value = Buffer.alloc(64);
        if (this.isLeft(leafKey, depth)) {
            this.hashBuffer
                .fill(node.hash, 0, 32)
                .fill(siblingHash, 32)
                .copy(value);
        }
        else {
            this.hashBuffer
                .fill(siblingHash, 0, 32)
                .fill(node.hash, 32)
                .copy(value);
        }
        return this.createNode(this.hashFunction(value), value, this.getNodeKey(leafKey, depth));
    }
    async populateZeroHashesAndRoot(rootHash) {
        const hashes = [
            this.hashFunction(SparseMerkleTreeImpl.emptyBuffer),
        ];
        for (let i = 1; i < this.height; i++) {
            hashes[i] = this.hashFunction(this.hashBuffer.fill(hashes[i - 1], 0, 32).fill(hashes[i - 1], 32));
        }
        this.zeroHashes = hashes.reverse();
        if (!!rootHash) {
            log.info(`Attempting to initialize SMT with root hash ${rootHash.toString('hex')} and DB ID: ${this.getNodeIDFromHashAndKey(rootHash, core_utils_1.ZERO).toString('hex')}`);
            this.root = await this.getNode(rootHash, core_utils_1.ZERO);
        }
        if (!this.root) {
            this.root = this.createNode(rootHash || this.zeroHashes[0], undefined, core_utils_1.ZERO);
            log.info(`Initialized Sparse Merkle Tree with root ${(rootHash || this.zeroHashes[0]).toString('hex')}`);
        }
        else {
            log.info(`Initialized Sparse Merkle Tree with root hash [${this.root.hash.toString('hex')}] and root value: [${!!this.root.value ? this.root.value.toString('hex') : 'undefined'}]`);
        }
    }
    createNode(hash, value, key) {
        return { hash, value, key };
    }
    isLeft(key, depth) {
        return key
            .shiftLeft(depth)
            .shiftRight(this.height - 2)
            .mod(core_utils_1.TWO)
            .equals(core_utils_1.ZERO);
    }
    getNodeKey(leafKey, depth) {
        return leafKey.shiftRight(this.height - depth - 1);
    }
    getNodeID(node) {
        return this.getNodeIDFromHashAndKey(node.hash, node.key);
    }
    getNodeIDFromHashAndKey(nodeHash, nodeKey) {
        return this.hashFunction(this.hashBuffer
            .fill(nodeHash, 0, 32)
            .fill(this.hashFunction(nodeKey.toBuffer(core_utils_1.BIG_ENDIAN)), 32));
    }
    getNodePutBatch(nodes) {
        return nodes.map((n) => {
            const nodeId = this.getNodeID(n);
            log.debug(`Setting nodeID: ${nodeId.toString('hex')} to value: ${n.value.toString('hex')} (hash: ${this.hashFunction(n.value).toString('hex')}`);
            return {
                type: db_interface_1.PUT_BATCH_TYPE,
                key: this.getNodeID(n),
                value: n.value,
            };
        });
    }
}
exports.SparseMerkleTreeImpl = SparseMerkleTreeImpl;
SparseMerkleTreeImpl.emptyBuffer = Buffer.alloc(32).fill('\x00');
SparseMerkleTreeImpl.siblingBuffer = Buffer.alloc(1).fill('\x00');
SparseMerkleTreeImpl.lockKey = 'lock';
//# sourceMappingURL=merkle-tree.js.map