"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseDB = exports.getLevelInstance = exports.defaultLevelOptions = exports.newInMemoryDB = exports.DEFAULT_PREFIX_LENGTH = void 0;
const core_utils_1 = require("@eth-optimism/core-utils");
const level = require('level');
const memdown_1 = require("memdown");
const iterator_1 = require("./iterator");
const bucket_1 = require("./bucket");
const range_bucket_1 = require("./range-bucket");
const log = core_utils_1.getLogger('db');
exports.DEFAULT_PREFIX_LENGTH = 3;
const isNotFound = (err) => {
    if (!err) {
        return false;
    }
    return (err.notFound ||
        err.type === 'NotFoundError' ||
        /not\s*found/i.test(err.message));
};
let memId = core_utils_1.ZERO;
exports.newInMemoryDB = (prefixLength = 256, options) => {
    memId = memId.add(core_utils_1.ONE);
    return new BaseDB(new memdown_1.default(`newInMemoryDB/${memId.toString()}`), prefixLength);
};
exports.defaultLevelOptions = {
    keyEncoding: 'binary',
    valueEncoding: 'binary',
};
exports.getLevelInstance = (path, options = exports.defaultLevelOptions) => {
    return level(path, options);
};
class BaseDB {
    constructor(db, prefixLength = exports.DEFAULT_PREFIX_LENGTH) {
        this.db = db;
        this.prefixLength = prefixLength;
    }
    async open(options) {
        try {
            await new Promise((resolve, reject) => {
                this.db.open(options, (err) => {
                    if (err) {
                        reject(err);
                        log.error(`Error opening DB: ${err.message}, ${err.stack}`);
                        return;
                    }
                    resolve();
                });
            });
        }
        catch (err) {
            throw err;
        }
    }
    async close() {
        try {
            await new Promise((resolve, reject) => {
                this.db.close((err) => {
                    if (err) {
                        reject(err);
                        log.error(`Error closing DB: ${err.message}, ${err.stack}`);
                        return;
                    }
                    resolve();
                });
            });
        }
        catch (err) {
            throw err;
        }
    }
    async get(key) {
        return new Promise((resolve, reject) => {
            this.db.get(key, (err, value) => {
                if (err) {
                    if (isNotFound(err)) {
                        log.debug(`Key ${key.toString('hex')} not found.`);
                        resolve(null);
                        return;
                    }
                    reject(err);
                    log.error(`Error getting key ${key.toString('hex')}: ${err.message}, ${err.stack}`);
                    return;
                }
                resolve(value);
            });
        });
    }
    async put(key, value) {
        return new Promise((resolve, reject) => {
            this.db.put(key, value, (err) => {
                if (err) {
                    reject(err);
                    log.error(`Error putting key / value ${key.toString('hex')} / ${value.toString('hex')}: ${err.message}, ${err.stack}`);
                    return;
                }
                resolve();
            });
        });
    }
    async del(key) {
        return new Promise((resolve, reject) => {
            this.db.del(key, (err) => {
                if (err) {
                    reject(err);
                    log.error(`Error deleting key ${key.toString('hex')}: ${err.message}, ${err.stack}`);
                    return;
                }
                resolve();
            });
        });
    }
    async has(key) {
        try {
            await this.get(key);
            return true;
        }
        catch (err) {
            log.error(`Error checking key existence: key ${key.toString('hex')}: ${err.message}, ${err.stack}`);
            return false;
        }
    }
    async batch(operations) {
        return new Promise((resolve, reject) => {
            this.db.batch(operations, (err) => {
                if (err) {
                    reject(err);
                    log.error(`Error executing batch operation: ${err.message}, ${err.stack}`);
                    return;
                }
                resolve();
            });
        });
    }
    iterator(options) {
        return new iterator_1.BaseIterator(this, options);
    }
    bucket(prefix) {
        return new bucket_1.BaseBucket(this, core_utils_1.bufferUtils.padLeft(prefix, this.prefixLength));
    }
    rangeBucket(prefix) {
        return new range_bucket_1.BaseRangeBucket(this, core_utils_1.bufferUtils.padLeft(prefix, this.prefixLength));
    }
}
exports.BaseDB = BaseDB;
//# sourceMappingURL=db.js.map