"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EthereumBlockProcessor = void 0;
const core_utils_1 = require("@eth-optimism/core-utils");
const log = core_utils_1.getLogger('ethereum-block-processor');
const blockKey = Buffer.from('latestBlock');
class EthereumBlockProcessor {
    constructor(db, earliestBlock = 0, confirmsUntilFinal = 1) {
        this.db = db;
        this.earliestBlock = earliestBlock;
        this.confirmsUntilFinal = confirmsUntilFinal;
        this.subscriptions = new Set();
        this.currentFinalizedBlockNumber = 0;
        this.syncInProgress = false;
        this.syncCompleted = false;
        if (earliestBlock < 0) {
            throw Error('Earliest block must be >= 0');
        }
    }
    async subscribe(provider, handler, syncPastBlocks = true) {
        this.subscriptions.add(handler);
        provider.on('block', async (blockNumber) => {
            try {
                const finalizedBlockNumber = this.getBlockFinalizedBy(blockNumber);
                if (finalizedBlockNumber < this.earliestBlock) {
                    log.debug(`Received block [${blockNumber}] which finalizes a block ${finalizedBlockNumber}, before earliest block [${this.earliestBlock}]. Ignoring...`);
                    return;
                }
                log.debug(`Block [${blockNumber}] was mined! Finalizing block ${finalizedBlockNumber}`);
                await this.fetchAndDisseminateBlock(provider, finalizedBlockNumber);
                this.currentFinalizedBlockNumber = finalizedBlockNumber;
                if (!syncPastBlocks || this.syncCompleted) {
                    await this.storeLastProcessedBlockNumber(this.currentFinalizedBlockNumber);
                }
            }
            catch (e) {
                core_utils_1.logError(log, `Error thrown processing block ${blockNumber}, finalizing block ${this.getBlockFinalizedBy(blockNumber)}. Exiting since throwing will not be caught.`, e);
                process.exit(1);
            }
        });
        if (syncPastBlocks) {
            if (this.syncCompleted) {
                await handler.onSyncCompleted();
                return;
            }
            if (!this.syncInProgress) {
                this.syncInProgress = true;
                await this.syncBlocks(provider);
            }
        }
    }
    async fetchAndDisseminateBlock(provider, blockNumber) {
        log.debug(`Fetching block [${blockNumber}].`);
        const block = await provider.getBlock(blockNumber, true);
        log.debug(`Received block: ${block.number}.`);
        this.subscriptions.forEach((h) => {
            try {
                h.handle(block);
            }
            catch (e) {
                core_utils_1.logError(log, `Error in subscriber handling block number ${blockNumber}. Re-throwing because we cannot proceed skipping a block.`, e);
                throw e;
            }
        });
    }
    async syncBlocks(provider) {
        log.debug(`Syncing blocks.`);
        const lastSynced = await this.getLastSyncedBlockNumber();
        const syncStart = Math.max(lastSynced + 1, this.earliestBlock);
        const mostRecentBlock = await this.getBlockNumber(provider);
        const mostRecentFinalBlock = this.getBlockFinalizedBy(mostRecentBlock);
        log.debug(`Starting sync with block ${syncStart}. Last synced: ${lastSynced}, earliest block: ${this.earliestBlock}, most recent un-finalized block: ${mostRecentBlock}, most recent finalized block: ${mostRecentFinalBlock}.`);
        if (mostRecentFinalBlock < syncStart) {
            log.debug(`Up to date, not syncing.`);
            this.finishSync(mostRecentFinalBlock, mostRecentFinalBlock);
            return;
        }
        for (let i = syncStart; i <= mostRecentFinalBlock; i++) {
            try {
                log.debug(`Syncing past blocks: Fetching and disseminating block ${i}`);
                await this.fetchAndDisseminateBlock(provider, i);
            }
            catch (e) {
                core_utils_1.logError(log, `Error fetching and disseminating block. Retrying...`, e);
                i--;
                continue;
            }
            await this.storeLastProcessedBlockNumber(i);
        }
        this.finishSync(syncStart, mostRecentFinalBlock);
    }
    finishSync(syncStart, currentBlock) {
        this.syncCompleted = true;
        this.syncInProgress = false;
        if (syncStart !== currentBlock) {
            log.debug(`Synced from block [${syncStart}] to [${currentBlock}]!`);
        }
        else {
            log.debug(`No sync necessary. Last processed and current block are the same block number: ${currentBlock}`);
        }
        for (const callback of this.subscriptions) {
            callback.onSyncCompleted().catch((e) => {
                core_utils_1.logError(log, 'Error calling Block sync callback', e);
            });
        }
    }
    async getBlockNumber(provider) {
        if (this.currentFinalizedBlockNumber === 0) {
            this.currentFinalizedBlockNumber = await provider.getBlockNumber();
        }
        log.debug(`Current block number: ${this.currentFinalizedBlockNumber}`);
        return this.currentFinalizedBlockNumber;
    }
    getBlockFinalizedBy(finalizingBlock) {
        if (this.confirmsUntilFinal <= 1) {
            return finalizingBlock;
        }
        return finalizingBlock - (this.confirmsUntilFinal - 1);
    }
    async getLastSyncedBlockNumber() {
        const lastSyncedBlockBuffer = await this.db.get(blockKey);
        return !!lastSyncedBlockBuffer
            ? parseInt(lastSyncedBlockBuffer.toString(), 10)
            : this.earliestBlock - 1;
    }
    async storeLastProcessedBlockNumber(blockNumber) {
        try {
            await this.db.put(blockKey, Buffer.from(blockNumber.toString()));
        }
        catch (e) {
            core_utils_1.logError(log, `Error storing most recent block received [${blockNumber}]!`, e);
        }
    }
}
exports.EthereumBlockProcessor = EthereumBlockProcessor;
//# sourceMappingURL=ethereum-block-processor.js.map