"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EthereumEventProcessor = void 0;
const core_utils_1 = require("@eth-optimism/core-utils");
const log = core_utils_1.getLogger('ethereum-event-processor');
class EthereumEventProcessor {
    constructor(db, earliestBlock = 0, confirmsUntilFinal = 1) {
        this.db = db;
        this.earliestBlock = earliestBlock;
        this.confirmsUntilFinal = confirmsUntilFinal;
        this.subscriptions = new Map();
        this.currentBlockNumber = 0;
        this.syncStatuses = new Map();
    }
    async subscribe(contract, eventName, handler, syncPastEvents = true) {
        const eventId = EthereumEventProcessor.getEventID(contract.address, eventName);
        log.debug(`Received subscriber for event ${eventName}, ID: ${eventId}`);
        if (!this.subscriptions.has(eventId)) {
            this.subscriptions.set(eventId, new Set([handler]));
        }
        else {
            this.subscriptions.get(eventId).add(handler);
            return;
        }
        contract.on(contract.filters[eventName](), async (...data) => {
            log.debug(`Received live event: ${JSON.stringify(data)}`);
            const ethersEvent = data[data.length - 1];
            const event = this.createEventFromEthersEvent(ethersEvent);
            if (this.confirmsUntilFinal > 1) {
                log.debug(`Waiting for ${this.confirmsUntilFinal} confirms before disseminating event: ${JSON.stringify(data)}`);
                try {
                    await contract.provider.waitForTransaction(event.transactionHash, this.confirmsUntilFinal);
                }
                catch (e) {
                    core_utils_1.logError(log, `Error waiting for ${this.confirmsUntilFinal} confirms on event ${JSON.stringify(data)}`, e);
                    return;
                }
            }
            await this.handleEvent(event);
            try {
                await this.db.put(Buffer.from(event.eventID), Buffer.from(ethersEvent.blockNumber.toString(10)));
            }
            catch (e) {
                core_utils_1.logError(log, `Error storing most recent events block [${ethersEvent.blockNumber}]!`, e);
            }
        });
        if (syncPastEvents) {
            if (!this.syncStatuses.has(eventId)) {
                this.syncStatuses.set(eventId, {
                    syncInProgress: true,
                    syncCompleted: false,
                });
                await this.syncPastEvents(contract, eventName, eventId);
                return;
            }
            const syncStatus = this.syncStatuses.get(eventId);
            if (syncStatus.syncCompleted) {
                await handler.onSyncCompleted(eventName);
            }
        }
    }
    static getEventID(address, eventName) {
        return core_utils_1.Md5Hash(`${address}${eventName}`);
    }
    async syncPastEvents(contract, eventName, eventId) {
        log.debug(`Syncing events for event ${eventName}`);
        const blockNumber = await this.getBlockNumber(contract.provider);
        const lastSyncedBlockBuffer = await this.db.get(Buffer.from(eventId));
        const lastSyncedNumber = !!lastSyncedBlockBuffer
            ? parseInt(lastSyncedBlockBuffer.toString(), 10)
            : this.earliestBlock;
        if (blockNumber === lastSyncedNumber) {
            log.debug(`Up to date, not syncing.`);
            this.finishSync(eventId, eventName, 0);
            return;
        }
        const filter = contract.filters[eventName]();
        filter.fromBlock = lastSyncedNumber + 1;
        filter.toBlock = 'latest';
        const logs = await contract.provider.getLogs(filter);
        const events = logs.map((l) => {
            const logDesc = contract.interface.parseLog(l);
            return EthereumEventProcessor.createEventFromLogDesc(l, logDesc, eventId);
        });
        for (const event of events) {
            await this.handleEvent(event);
        }
        this.finishSync(eventId, eventName, events.length);
    }
    finishSync(eventId, eventName, numEvents) {
        const status = this.syncStatuses.get(eventId);
        status.syncCompleted = true;
        status.syncInProgress = false;
        log.debug(`Synced events for event ${eventName}, ${eventId}. Found ${numEvents} events`);
        for (const subscription of this.subscriptions.get(eventId)) {
            subscription.onSyncCompleted(eventId).catch((e) => {
                core_utils_1.logError(log, 'Error calling EthereumEvent sync callback', e);
            });
        }
    }
    async handleEvent(event) {
        log.debug(`Handling event ${JSON.stringify(event)}`);
        const subscribers = this.subscriptions.get(event.eventID);
        subscribers.forEach((s) => {
            try {
                s.handle(event);
            }
            catch (e) {
            }
        });
    }
    async getBlockNumber(provider) {
        if (this.currentBlockNumber === 0) {
            this.currentBlockNumber = await provider.getBlockNumber();
        }
        return this.currentBlockNumber;
    }
    static createEventFromLogDesc(logObj, logDesc, eventID) {
        const values = EthereumEventProcessor.getLogValues(logDesc.values);
        return {
            eventID,
            name: logDesc.name,
            signature: logDesc.signature,
            values,
            blockHash: logObj.blockHash,
            blockNumber: logObj.blockNumber,
            transactionHash: logObj.transactionHash,
        };
    }
    createEventFromEthersEvent(event) {
        const values = EthereumEventProcessor.getLogValues(event.args);
        return {
            eventID: EthereumEventProcessor.getEventID(event.address, event.event),
            name: event.event,
            signature: event.eventSignature,
            values,
            blockHash: event.blockHash,
            blockNumber: event.blockNumber,
            transactionHash: event.transactionHash,
        };
    }
    static getLogValues(logArgs) {
        const values = Object.assign({}, logArgs);
        for (let i = 0; i < logArgs['length']; i++) {
            delete values[i.toString()];
        }
        delete values['length'];
        return values;
    }
}
exports.EthereumEventProcessor = EthereumEventProcessor;
//# sourceMappingURL=ethereum-event-processor.js.map