"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseRangeIterator = exports.BaseIterator = void 0;
const defaultIteratorOptions = {
    reverse: false,
    limit: -1,
    keys: true,
    values: true,
    keyAsBuffer: true,
    valueAsBuffer: true,
    prefix: Buffer.from(''),
};
class BaseIterator {
    constructor(db, options = {}) {
        this.db = db;
        this.prefix = options.prefix || defaultIteratorOptions.prefix;
        if (options.gte !== undefined) {
            options.gte = this.addPrefix(options.gte);
        }
        if (options.lte !== undefined) {
            options.lte = this.addPrefix(options.lte);
        }
        if (options.gt !== undefined) {
            options.gt = this.addPrefix(options.gt);
        }
        if (options.lt !== undefined) {
            options.lt = this.addPrefix(options.lt);
        }
        this.options = Object.assign(Object.assign({}, defaultIteratorOptions), options);
    }
    async next() {
        this.start();
        const { key, value } = await new Promise((resolve, reject) => {
            this.iterator.next((err, k, v) => {
                if (err) {
                    reject(err);
                    return;
                }
                resolve({ key: k, value: v });
            });
        });
        if (key === undefined && value === undefined) {
            this.cleanup();
        }
        return { key: this.removePrefix(key), value };
    }
    async seek(target) {
        this.start();
        this.iterator.seek(this.addPrefix(target));
    }
    async each(cb) {
        while (!this.finished) {
            const { key, value } = await this.next();
            if (this.finished) {
                return this.end();
            }
            let result;
            try {
                result = cb(key, value);
                if (result instanceof Promise) {
                    result = await result;
                }
            }
            catch (err) {
                await this.end();
                throw err;
            }
            if (result === false) {
                return this.end();
            }
        }
        return this.end();
    }
    async items() {
        const items = [];
        await this.each((key, value) => {
            return items.push({
                key,
                value,
            });
        });
        return items;
    }
    async keys() {
        const items = await this.items();
        return items.map((item) => {
            return item.key;
        });
    }
    async values() {
        const items = await this.items();
        return items.map((item) => {
            return item.value;
        });
    }
    async end() {
        if (!this.iterator) {
            try {
                this.start();
            }
            catch (err) {
                throw err;
            }
        }
        this.cleanup();
        await new Promise((resolve, reject) => {
            this.iterator.end((err) => {
                if (err) {
                    reject(err);
                    return;
                }
                resolve();
            });
        });
    }
    start() {
        if (this.iterator !== undefined) {
            return;
        }
        this.iterator = this.db.db.iterator(this.options);
    }
    cleanup() {
        this.finished = true;
    }
    addPrefix(value) {
        return value ? Buffer.concat([this.prefix, value]) : value;
    }
    removePrefix(value) {
        return value ? value.slice(this.prefix.length) : value;
    }
}
exports.BaseIterator = BaseIterator;
class BaseRangeIterator extends BaseIterator {
    constructor(db, options = {}, resultToRange) {
        super(db, options);
        this.resultToRange = resultToRange;
    }
    async nextRange() {
        const res = await this.next();
        if (typeof res.key === 'undefined') {
            return;
        }
        return this.resultToRange(res);
    }
}
exports.BaseRangeIterator = BaseRangeIterator;
//# sourceMappingURL=iterator.js.map