"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseKey = void 0;
const types_1 = require("./types");
const utils_1 = require("./utils");
class BaseKey {
    constructor(id, ops = []) {
        this.ops = [];
        this.size = 0;
        this.index = -1;
        this.id = utils_1.makeID(id);
        for (let i = 0; i < ops.length; i++) {
            const name = ops[i];
            if (!(name in types_1.types)) {
                throw new Error(`Invalid type name: ${name}.`);
            }
            const op = types_1.types[name];
            if (op.dynamic) {
                if (this.index === -1) {
                    this.index = i;
                }
            }
            else {
                this.size += op.size();
            }
            this.ops.push(op);
        }
    }
    getSize(args) {
        utils_1.assert(args.length === this.ops.length);
        let size = 1 + this.size;
        if (this.index === -1) {
            return size;
        }
        for (let i = this.index; i < args.length; i++) {
            const op = this.ops[i];
            const arg = args[i];
            if (op.dynamic) {
                size += op.size(arg);
            }
        }
        return size;
    }
    encode(args = []) {
        utils_1.assert(args.length === this.ops.length);
        const size = this.getSize(args);
        const key = Buffer.allocUnsafe(size);
        key[0] = this.id;
        let offset = 1;
        for (let i = 0; i < this.ops.length; i++) {
            const op = this.ops[i];
            const arg = args[i];
            offset += op.write(key, arg, offset);
        }
        return key;
    }
    decode(key) {
        if (this.ops.length === 0) {
            return key;
        }
        if (key.length === 0 || key[0] !== this.id) {
            throw new Error('Key prefix mismatch.');
        }
        const args = [];
        let offset = 1;
        for (const op of this.ops) {
            const arg = op.read(key, offset);
            offset += op.size(arg);
            args.push(arg);
        }
        return args;
    }
    min(args = []) {
        for (let i = args.length; i < this.ops.length; i++) {
            const op = this.ops[i];
            args.push(op.min);
        }
        return this.encode(args);
    }
    max(args = []) {
        for (let i = args.length; i < this.ops.length; i++) {
            const op = this.ops[i];
            args.push(op.max);
        }
        return this.encode(args);
    }
}
exports.BaseKey = BaseKey;
//# sourceMappingURL=key.js.map