"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.types = void 0;
const utils_1 = require("./utils");
exports.types = {
    char: {
        min: '\x00',
        max: '\xff',
        dynamic: false,
        size() {
            return 1;
        },
        read(k, o) {
            utils_1.assertLen(o + 1 <= k.length);
            return String.fromCharCode(k[o]);
        },
        write(k, v, o) {
            utils_1.assertType(typeof v === 'string');
            utils_1.assertType(v.length === 1);
            utils_1.assertLen(o + 1 <= k.length);
            k[o] = v.charCodeAt(0);
            return 1;
        },
    },
    uint8: {
        min: 0,
        max: 0xff,
        dynamic: false,
        size() {
            return 1;
        },
        read(k, o) {
            utils_1.assertLen(o + 1 <= k.length);
            return k[o];
        },
        write(k, v, o) {
            utils_1.assertType((v & 0xff) === v);
            utils_1.assertLen(o + 1 <= k.length);
            k[o] = v;
            return 1;
        },
    },
    uint16: {
        min: 0,
        max: 0xffff,
        dynamic: false,
        size() {
            return 2;
        },
        read(k, o) {
            utils_1.assertLen(o + 2 <= k.length);
            return k.readUInt16BE(o);
        },
        write(k, v, o) {
            utils_1.assertType((v & 0xffff) === v);
            utils_1.assertLen(o + 2 <= k.length);
            k.writeUInt16BE(v, o);
            return 2;
        },
    },
    uint32: {
        min: 0,
        max: 0xffffffff,
        dynamic: false,
        size() {
            return 4;
        },
        read(k, o) {
            utils_1.assertLen(o + 4 <= k.length);
            return k.readUInt32BE(o);
        },
        write(k, v, o) {
            utils_1.assertType(v >>> 0 === v);
            utils_1.assertLen(o + 4 <= k.length);
            k.writeUInt32BE(v, o);
            return 4;
        },
    },
    uint256: {
        min: Buffer.alloc(32, 0x00),
        max: Buffer.alloc(32, 0xff),
        dynamic: false,
        size() {
            return 32;
        },
        read(k, o) {
            utils_1.assertLen(o + 32 <= k.length);
            return k.slice(o, o + 32);
        },
        write(k, v, o) {
            utils_1.assertType(Buffer.isBuffer(v));
            utils_1.assertType(v.copy(k, o) === 32);
            return 32;
        },
    },
    buffer: {
        min: utils_1.BUFFER_MIN,
        max: utils_1.BUFFER_MAX,
        dynamic: true,
        size(v) {
            return utils_1.sizeBuffer(v);
        },
        read(k, o) {
            return utils_1.readBuffer(k, o);
        },
        write(k, v, o) {
            return utils_1.writeBuffer(k, v, o);
        },
    },
    hex: {
        min: utils_1.BUFFER_MIN.toString('hex'),
        max: utils_1.BUFFER_MAX.toString('hex'),
        dynamic: true,
        size(v) {
            return utils_1.sizeString(v, 'hex');
        },
        read(k, o) {
            return utils_1.readString(k, o, 'hex');
        },
        write(k, v, o) {
            return utils_1.writeString(k, v, o, 'hex');
        },
    },
    ascii: {
        min: utils_1.BUFFER_MIN.toString('binary'),
        max: utils_1.BUFFER_MAX.toString('binary'),
        dynamic: true,
        size(v) {
            return utils_1.sizeString(v, 'binary');
        },
        read(k, o) {
            return utils_1.readString(k, o, 'binary');
        },
        write(k, v, o) {
            return utils_1.writeString(k, v, o, 'binary');
        },
    },
    utf8: {
        min: utils_1.BUFFER_MIN.toString('utf8'),
        max: utils_1.BUFFER_MAX.toString('utf8'),
        dynamic: true,
        size(v) {
            return utils_1.sizeString(v, 'utf8');
        },
        read(k, o) {
            return utils_1.readString(k, o, 'utf8');
        },
        write(k, v, o) {
            return utils_1.writeString(k, v, o, 'utf8');
        },
    },
    hash160: {
        min: Buffer.alloc(20, 0x00),
        max: Buffer.alloc(20, 0xff),
        dynamic: false,
        size() {
            return 20;
        },
        read(k, o) {
            utils_1.assertLen(o + 20 <= k.length);
            return k.slice(o, o + 20);
        },
        write(k, v, o) {
            utils_1.assertType(Buffer.isBuffer(v));
            utils_1.assertType(v.copy(k, o) === 20);
            return 20;
        },
    },
    hash256: {
        min: Buffer.alloc(32, 0x00),
        max: Buffer.alloc(32, 0xff),
        dynamic: false,
        size() {
            return 32;
        },
        read(k, o) {
            utils_1.assertLen(o + 32 <= k.length);
            return k.slice(o, o + 32);
        },
        write(k, v, o) {
            utils_1.assertType(Buffer.isBuffer(v));
            utils_1.assertType(v.copy(k, o) === 32);
            return 32;
        },
    },
    hash: {
        min: Buffer.alloc(1, 0x00),
        max: Buffer.alloc(64, 0xff),
        dynamic: true,
        size(v) {
            utils_1.assertType(Buffer.isBuffer(v));
            return 1 + v.length;
        },
        read(k, o) {
            utils_1.assertLen(o + 1 <= k.length);
            utils_1.assertLen(k[o] >= 1 && k[o] <= 64);
            utils_1.assertLen(o + 1 + k[o] <= k.length);
            return k.slice(o + 1, o + 1 + k[o]);
        },
        write(k, v, o) {
            utils_1.assertType(Buffer.isBuffer(v));
            utils_1.assertType(v.length >= 1 && v.length <= 64);
            utils_1.assertLen(o + 1 <= k.length);
            k[o] = v.length;
            utils_1.assertType(v.copy(k, o + 1) === v.length);
            return 1 + v.length;
        },
    },
    hhash160: {
        min: Buffer.alloc(20, 0x00),
        max: Buffer.alloc(20, 0xff),
        dynamic: false,
        size() {
            return 20;
        },
        read(k, o) {
            utils_1.assertLen(o + 20 <= k.length);
            return k.toString('hex', o, o + 20);
        },
        write(k, v, o) {
            utils_1.assertType(utils_1.writeHex(k, v, o) === 20);
            return 20;
        },
    },
    hhash256: {
        min: Buffer.alloc(32, 0x00),
        max: Buffer.alloc(32, 0xff),
        dynamic: false,
        size() {
            return 32;
        },
        read(k, o) {
            utils_1.assertLen(o + 32 <= k.length);
            return k.toString('hex', o, o + 32);
        },
        write(k, v, o) {
            utils_1.assertType(utils_1.writeHex(k, v, o) === 32);
            return 32;
        },
    },
    hhash: {
        min: Buffer.alloc(1, 0x00),
        max: Buffer.alloc(64, 0xff),
        dynamic: true,
        size(v) {
            return 1 + utils_1.sizeHex(v);
        },
        read(k, o) {
            utils_1.assertLen(o + 1 <= k.length);
            utils_1.assertLen(k[o] >= 1 && k[o] <= 64);
            utils_1.assertLen(o + 1 + k[o] <= k.length);
            return k.toString('hex', o + 1, o + 1 + k[o]);
        },
        write(k, v, o) {
            const size = utils_1.sizeHex(v);
            utils_1.assertType(size >= 1 && size <= 64);
            utils_1.assertLen(o + 1 <= k.length);
            k[o] = size;
            utils_1.assertType(utils_1.writeHex(k, v, o + 1) === size);
            return 1 + size;
        },
    },
};
//# sourceMappingURL=types.js.map