"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseQueuedPersistedProcessor = void 0;
const core_utils_1 = require("@eth-optimism/core-utils");
const AsyncLock = require("async-lock");
const log = core_utils_1.getLogger('base-persisted-queue');
const lockKey = 'lock_key';
class BaseQueuedPersistedProcessor {
    constructor(processingDataService, persistenceKey, startIndex = 0, retrySleepDelayMillis = 1000) {
        this.processingDataService = processingDataService;
        this.persistenceKey = persistenceKey;
        this.startIndex = startIndex;
        this.retrySleepDelayMillis = retrySleepDelayMillis;
        this.processingLock = new AsyncLock();
    }
    async add(index, item) {
        await this.processingDataService.persistItem(index, await this.serializeItem(item), this.persistenceKey);
        this.handleIfReady(index, item);
    }
    async markProcessed(index) {
        this.log(`Marking index ${index} as processed for processor ${this.persistenceKey}`);
        await this.processingDataService.updateToProcessed(index, this.persistenceKey);
        setTimeout(() => {
            this.handleIfExists(index + 1);
        }, 0);
    }
    async getLastIndexProcessed() {
        const last = await this.processingDataService.getLastIndexProcessed(this.persistenceKey);
        return Math.max(last, this.startIndex - 1);
    }
    async init() {
        const lastProcessed = await this.getLastIndexProcessed();
        return this.handleIfExists(lastProcessed + 1);
    }
    log(msg, error = false, e) {
        const message = `[${this.persistenceKey}] ${msg}`;
        if (error) {
            if (!!e) {
                core_utils_1.logError(log, message, e);
            }
            else {
                log.error(message);
            }
        }
        else {
            log.debug(message);
        }
    }
    logError(msg, e) {
        this.log(msg, true, e);
    }
    async handleIfReady(index, item) {
        const shouldRetry = await this.processingLock.acquire(lockKey, async () => {
            try {
                const lastProcessed = await this.getLastIndexProcessed();
                if (lastProcessed !== index - 1) {
                    this.log(`Told to process ${index} but last index processed was ${lastProcessed}, so will not process.`);
                    return false;
                }
                this.log(`Handling index ${index}.`);
                await this.handleNextItem(index, item);
            }
            catch (e) {
                this.log(`Error handling item ${index}. Going to retry.`, true, e);
                return true;
            }
        });
        if (!shouldRetry) {
            return;
        }
        await core_utils_1.sleep(this.retrySleepDelayMillis);
        return this.handleIfReady(index, item);
    }
    async handleIfExists(index) {
        const item = await this.processingDataService.fetchItem(index, this.persistenceKey);
        if (!item) {
            this.log(`Index ${index} not yet present. Waiting for its arrival to process it.`);
            return;
        }
        else if (!!item.processed) {
            this.log(`Index ${index} already processed. Attempting to process ${index + 1}`);
            setTimeout(() => {
                this.handleIfExists(index + 1);
            }, 0);
            return;
        }
        return this.handleIfReady(index, await this.deserializeItem(item.data));
    }
}
exports.BaseQueuedPersistedProcessor = BaseQueuedPersistedProcessor;
//# sourceMappingURL=queued-persisted-processor.js.map