"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InMemoryProcessingDataService = exports.DefaultSequentialProcessingDataService = void 0;
const core_utils_1 = require("@eth-optimism/core-utils");
const log = core_utils_1.getLogger('sequential-processing-data-service');
const doesColumnDataExist = (queryRes, rowIndex, columnName) => {
    return (!!queryRes &&
        queryRes.length > rowIndex &&
        !!queryRes[rowIndex] &&
        queryRes[rowIndex][columnName] !== null &&
        queryRes[rowIndex][columnName] !== undefined);
};
class DefaultSequentialProcessingDataService {
    constructor(rdb) {
        this.rdb = rdb;
    }
    async fetchItem(index, sequenceKey) {
        const res = await this.rdb.select(`SELECT data_to_process, processed
      FROM sequential_processing
      WHERE
        sequence_key = '${sequenceKey}'
        AND sequence_number = ${index}`);
        if (!doesColumnDataExist(res, 0, 'data_to_process')) {
            return undefined;
        }
        return {
            data: res[0]['data_to_process'],
            processed: !!res[0]['processed'],
        };
    }
    async getLastIndexProcessed(sequenceKey) {
        const res = await this.rdb.select(`SELECT MAX(sequence_number) as last_processed
      FROM sequential_processing
      WHERE 
        sequence_key = '${sequenceKey}'
        AND processed = TRUE`);
        if (!doesColumnDataExist(res, 0, 'last_processed')) {
            return -1;
        }
        return parseInt(res[0]['last_processed'], 10);
    }
    async persistItem(index, itemData, sequenceKey, processed = false) {
        try {
            await this.rdb.execute(`INSERT INTO sequential_processing(sequence_key, sequence_number, data_to_process, processed)
        VALUES('${sequenceKey}', ${index}, '${itemData}', ${processed ? 'TRUE' : 'FALSE'})
        ON CONFLICT ON CONSTRAINT sequential_processing_sequence_key_sequence_number_key DO UPDATE
        SET data_to_process = '${itemData}'`);
        }
        catch (e) {
            core_utils_1.logError(log, `[${sequenceKey}] Error persisting index ${index} data: ${itemData}.`, e);
            throw e;
        }
        log.debug(`[${sequenceKey}] Persisted item with index ${index}: ${itemData}`);
    }
    async updateToProcessed(index, sequenceKey) {
        await this.rdb.execute(`UPDATE sequential_processing
      SET processed = TRUE
      WHERE 
        sequence_key = '${sequenceKey}'
        AND sequence_number = ${index}`);
        log.debug(`[${sequenceKey}] index ${index} updated to processed.`);
    }
}
exports.DefaultSequentialProcessingDataService = DefaultSequentialProcessingDataService;
class InMemoryProcessingDataService {
    constructor() {
        this.items = new Map();
        this.lastProcessedIndex = new Map();
    }
    async updateToProcessed(index, sequenceKey) {
        const map = this.items.get(sequenceKey);
        if (!map) {
            throw Error(`Tried to update processed for a sequence key that contains no items (${sequenceKey})`);
        }
        const item = map.get(index);
        if (!item) {
            throw Error(`Tried to updated index ${index} which does not exist.`);
        }
        item.processed = true;
        this.lastProcessedIndex.set(sequenceKey, index);
    }
    async persistItem(index, data, sequenceKey, processed = false) {
        if (!this.items.has(sequenceKey)) {
            this.items.set(sequenceKey, new Map());
        }
        if (!this.items.get(sequenceKey).has(index)) {
            this.items.get(sequenceKey).set(index, { processed, data });
        }
    }
    async fetchItem(index, sequenceKey) {
        if (!this.items.has(sequenceKey)) {
            return undefined;
        }
        return this.items.get(sequenceKey).get(index);
    }
    async getLastIndexProcessed(sequenceKey) {
        const index = this.lastProcessedIndex.get(sequenceKey);
        return index === undefined ? -1 : index;
    }
}
exports.InMemoryProcessingDataService = InMemoryProcessingDataService;
//# sourceMappingURL=sequential-processing-data-service.js.map