"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseRangeBucket = void 0;
const core_utils_1 = require("@eth-optimism/core-utils");
const types_1 = require("../types");
const iterator_1 = require("./iterator");
const utils_1 = require("./utils");
const log = core_utils_1.getLogger('range-db');
class BaseRangeBucket {
    constructor(db, prefix, keyLength = 16, endianness = core_utils_1.BIG_ENDIAN) {
        this.db = db;
        this.prefix = prefix;
        this.keyLength = keyLength;
        this.endianness = endianness;
    }
    addPrefix(value) {
        return value !== undefined
            ? Buffer.concat([this.prefix, value])
            : this.prefix;
    }
    addStartToValue(start, value) {
        return Buffer.concat([
            start.toBuffer(this.endianness, this.keyLength),
            value,
        ]);
    }
    getStartFromValue(value) {
        return value.slice(0, this.keyLength);
    }
    getDataFromValue(value) {
        return value.slice(this.keyLength);
    }
    bnToKey(bigNum) {
        const buf = bigNum.toBuffer(this.endianness, this.keyLength);
        return Buffer.concat([this.prefix, buf]);
    }
    isCorrectPrefix(key) {
        if (typeof key === 'undefined') {
            return false;
        }
        return Buffer.compare(this.prefix, key.slice(0, this.prefix.length)) === 0;
    }
    validateRange(start, end) {
        if (!start.lt(end)) {
            throw new Error('Start not less than end');
        }
        if (!start.gte(core_utils_1.ZERO)) {
            throw new Error('Start less than zero');
        }
    }
    resultToRange(result) {
        return {
            start: new core_utils_1.BigNumber(this.getStartFromValue(result.value)),
            end: new core_utils_1.BigNumber(result.key.slice(this.prefix.length)),
            value: this.getDataFromValue(result.value),
        };
    }
    async delBatchOps(start, end) {
        this.validateRange(start, end);
        const ranges = await this.get(start, end);
        const batchOps = [];
        for (const range of ranges) {
            batchOps.push({
                type: 'del',
                key: this.bnToKey(range.end),
            });
        }
        return { ranges, batchOps };
    }
    async put(start, end, value) {
        this.validateRange(start, end);
        log.debug('Putting range: [', start.toString(16), ',', end.toString(16), ') with value:', value);
        const { ranges, batchOps } = await this.delBatchOps(start, end);
        if (ranges.length > 0 && start.gt(ranges[0].start)) {
            batchOps.push({
                type: types_1.PUT_BATCH_TYPE,
                key: this.bnToKey(start),
                value: this.addStartToValue(ranges[0].start, ranges[0].value),
            });
        }
        if (ranges.length > 0 && ranges[ranges.length - 1].end.gt(end)) {
            batchOps.push({
                type: types_1.PUT_BATCH_TYPE,
                key: this.bnToKey(ranges[ranges.length - 1].end),
                value: this.addStartToValue(end, ranges[ranges.length - 1].value),
            });
        }
        batchOps.push({
            type: types_1.PUT_BATCH_TYPE,
            key: this.bnToKey(end),
            value: this.addStartToValue(start, value),
        });
        await this.db.batch(batchOps);
    }
    async del(start, end) {
        const { ranges, batchOps } = await this.delBatchOps(start, end);
        await this.db.batch(batchOps);
        return ranges;
    }
    async hasDataInRange(start, end) {
        return (await this.get(start, end)).length > 0;
    }
    async get(start, end) {
        this.validateRange(start, end);
        log.debug('Getting range: [', start.toString(16), ',', end.toString(16), ')');
        const it = this.db.iterator({
            gt: this.bnToKey(start),
            keyAsBuffer: true,
        });
        const ranges = [];
        let result = await it.next();
        if (!this.isCorrectPrefix(result.key)) {
            await it.end();
            return [];
        }
        const queryStart = this.bnToKey(start);
        const queryEnd = this.bnToKey(end);
        let resultStart = this.addPrefix(this.getStartFromValue(result.value));
        let resultEnd = result.key;
        while (utils_1.intersects(queryStart, queryEnd, resultStart, resultEnd) &&
            this.isCorrectPrefix(result.key)) {
            ranges.push(this.resultToRange(result));
            result = await it.next();
            if (typeof result.key === 'undefined') {
                break;
            }
            resultStart = this.addPrefix(this.getStartFromValue(result.value));
            resultEnd = result.key;
        }
        await it.end();
        return ranges;
    }
    iterator(options = {}) {
        const rangeIterator = new iterator_1.BaseRangeIterator(this.db, Object.assign(Object.assign({}, options), { prefix: this.addPrefix(options.prefix) }), (res) => this.resultToRange(res));
        return rangeIterator;
    }
    bucket(prefix) {
        return this.db.bucket(this.addPrefix(prefix));
    }
    rangeBucket(prefix) {
        return this.db.rangeBucket(this.addPrefix(prefix));
    }
}
exports.BaseRangeBucket = BaseRangeBucket;
//# sourceMappingURL=range-bucket.js.map