"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DefaultBlockRange = exports.doRangesSpanRange = exports.isRangeSubset = exports.rangesIntersect = exports.getOverlappingRange = exports.intersects = void 0;
const core_utils_1 = require("@eth-optimism/core-utils");
exports.intersects = (start1, end1, start2, end2) => {
    const maxStart = core_utils_1.bufferUtils.max(start1, start2);
    const minEnd = core_utils_1.bufferUtils.min(end1, end2);
    return core_utils_1.bufferUtils.lt(maxStart, minEnd);
};
exports.getOverlappingRange = (range1, range2) => {
    const start = core_utils_1.BigNumber.max(range1.start, range2.start);
    const end = core_utils_1.BigNumber.min(range1.end, range2.end);
    return start.lt(end) ? { start, end } : undefined;
};
exports.rangesIntersect = (range1, range2) => {
    return !!exports.getOverlappingRange(range1, range2);
};
exports.isRangeSubset = (subset, superset) => {
    return (subset !== undefined &&
        superset !== undefined &&
        subset.start.gte(superset.start) &&
        subset.end.lte(superset.end));
};
exports.doRangesSpanRange = (ranges, rangeToSpan) => {
    const sortedRanges = ranges.sort((a, b) => {
        return a.start.lt(b.start) ? -1 : a.start.eq(b.start) ? 0 : 1;
    });
    let lowestNotSpanned = rangeToSpan.start;
    for (const rangeElem of sortedRanges) {
        if (rangeElem.start.gt(lowestNotSpanned)) {
            return false;
        }
        lowestNotSpanned = rangeElem.end;
        if (lowestNotSpanned.gte(rangeToSpan.end)) {
            return true;
        }
    }
    return false;
};
class DefaultBlockRange {
    constructor(ranges = []) {
        this.ranges = ranges;
    }
    merge(other) {
        for (const range of other.ranges) {
            this.addRange(range);
        }
    }
    getOverlapping(range) {
        return this.ranges.reduce((overlap, existing) => {
            const overlapping = exports.getOverlappingRange(existing, range);
            if (!!overlapping) {
                overlap.push(Object.assign(Object.assign({}, existing), { start: overlapping.start, end: overlapping.end }));
            }
            return overlap;
        }, []);
    }
    addRange(range) {
        if (range.start.gte(range.end)) {
            throw new Error('Invalid range');
        }
        const toAdd = new DefaultBlockRange([range]);
        for (const overlap of this.getOverlapping(range)) {
            if (overlap.block.gt(range.block)) {
                toAdd.removeRange(overlap);
            }
            else {
                this.removeRange(overlap);
            }
        }
        this.ranges = this.ranges.concat(toAdd.ranges);
        this.sortRanges();
    }
    removeRange(range) {
        for (const overlap of this.getOverlapping(range)) {
            let removed;
            this.ranges = this.ranges.filter((r) => {
                if (r.start.lte(overlap.start) && r.end.gte(overlap.end)) {
                    removed = r;
                    return false;
                }
                return true;
            });
            if (removed.start.lt(overlap.start)) {
                this.ranges.push(Object.assign(Object.assign({}, removed), { end: overlap.start }));
            }
            if (removed.end.gt(overlap.end)) {
                this.ranges.push(Object.assign(Object.assign({}, removed), { start: overlap.end }));
            }
        }
        this.sortRanges();
    }
    incrementBlocks(range) {
        if (range.start.gte(range.end)) {
            throw new Error('Invalid range');
        }
        for (const existing of this.ranges) {
            const overlap = {
                start: core_utils_1.BigNumber.max(existing.start, range.start),
                end: core_utils_1.BigNumber.min(existing.end, range.end),
            };
            if (overlap.start.gte(overlap.end)) {
                continue;
            }
            this.addRange(Object.assign(Object.assign(Object.assign({}, existing), overlap), {
                block: existing.block.add(core_utils_1.ONE),
            }));
        }
    }
    sortRanges() {
        this.ranges = this.ranges.sort((a, b) => {
            return a.start.sub(b.start).toNumber();
        });
    }
}
exports.DefaultBlockRange = DefaultBlockRange;
//# sourceMappingURL=range.js.map