"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("../../setup");
const core_utils_1 = require("@eth-optimism/core-utils");
const ethereum_waffle_1 = require("ethereum-waffle");
const utils_1 = require("./utils");
const app_1 = require("../../../src/app");
const log = core_utils_1.getLogger('ethereum-block-processor-test', true);
const timeout = 25000;
describe('Block Subscription', () => {
    let provider;
    let wallets;
    let ownerWallet;
    let recipientWallet;
    let db;
    const sendAmount = 100;
    const initialSupply = 100000;
    let blockProcessor;
    let blockListener;
    let tokenContract;
    describe('Instant finalization', () => {
        beforeEach(async () => {
            provider = ethereum_waffle_1.createMockProvider();
            wallets = ethereum_waffle_1.getWallets(provider);
            ownerWallet = wallets[0];
            recipientWallet = wallets[1];
            log.debug(`Connection info: ${JSON.stringify(provider.connection)}`);
            tokenContract = await utils_1.deployTokenContract(ownerWallet, initialSupply);
            db = app_1.newInMemoryDB();
            blockProcessor = new app_1.EthereumBlockProcessor(db);
            blockListener = new utils_1.TestListener();
        });
        it('processes new blocks', async () => {
            await blockProcessor.subscribe(provider, blockListener, false);
            await tokenContract.transfer(ownerWallet.address, recipientWallet.address, sendAmount);
            const blocks = await blockListener.waitForReceive(1);
            blocks.length.should.equal(1);
            blocks[0].transactions.length.should.equal(1);
        }).timeout(timeout);
        it('processes old blocks', async () => {
            await blockProcessor.subscribe(provider, blockListener);
            const blocks = await blockListener.waitForSyncToComplete();
            blocks
                .map((x) => x.number)
                .sort()
                .should.deep.equal([0, 1], `Incorrect blocks received!`);
        }).timeout(timeout);
        it('honors earliest block', async () => {
            blockProcessor = new app_1.EthereumBlockProcessor(db, 1);
            await blockProcessor.subscribe(provider, blockListener);
            const blocks = await blockListener.waitForSyncToComplete();
            blocks.length.should.equal(1, 'There should only be one block synced!');
            blocks[0].number.should.equal(1, 'Block 1 should have been finalized!');
            blocks[0].transactions.length.should.equal(1, 'There should be 1 transactions in block 1');
            const deployToAddressEmpty = !blocks[0].transactions[0].to;
            deployToAddressEmpty.should.equal(true, 'The "to" address for the deploy tx should be null');
        }).timeout(timeout);
        it('processes blocks starting at 1', async () => {
            blockProcessor = new app_1.EthereumBlockProcessor(db, 1);
            await blockProcessor.subscribe(provider, blockListener);
            let blocks = await blockListener.waitForSyncToComplete();
            blocks.length.should.equal(1, 'Block 1 should have arrived');
            await tokenContract.transfer(ownerWallet.address, recipientWallet.address, sendAmount * 2);
            blocks = await blockListener.waitForReceive(2);
            blocks.length.should.equal(2, `Incorrect number of blocks received!`);
            blocks
                .map((x) => x.number)
                .sort()
                .should.deep.equal([1, 2], `Incorrect blocks received!`);
            blocks
                .filter((x) => x.number === 2)[0]
                .transactions.length.should.equal(1, `Tx Length incorrect!`);
        }).timeout(timeout);
        it('processes old and new blocks', async () => {
            await blockProcessor.subscribe(provider, blockListener);
            await tokenContract.transfer(ownerWallet.address, recipientWallet.address, sendAmount * 2);
            const blocks = await blockListener.waitForReceive(3);
            blocks.length.should.equal(3, `Incorrect number of blocks received!`);
            blocks
                .map((x) => x.number)
                .sort()
                .should.deep.equal([0, 1, 2], `Incorrect blocks received!`);
            blocks
                .filter((x) => x.number === 2)[0]
                .transactions.length.should.equal(1, `Tx Length incorrect!`);
        }).timeout(timeout);
    });
    describe('Delayed finalization', () => {
        const confirmsUntilFinal = 2;
        beforeEach(async () => {
            provider = ethereum_waffle_1.createMockProvider();
            wallets = ethereum_waffle_1.getWallets(provider);
            ownerWallet = wallets[0];
            recipientWallet = wallets[1];
            log.debug(`Connection info: ${JSON.stringify(provider.connection)}`);
            db = app_1.newInMemoryDB();
            blockProcessor = new app_1.EthereumBlockProcessor(db, 0, confirmsUntilFinal);
            blockListener = new utils_1.TestListener();
        });
        it('does not process un-finalized block', async () => {
            await blockProcessor.subscribe(provider, blockListener, false);
            const blocks = await blockListener.waitForReceive(1, 5000);
            blocks.length.should.equal(0);
        }).timeout(timeout);
        it('finalizes blocks after enough confirms', async () => {
            await blockProcessor.subscribe(provider, blockListener, false);
            tokenContract = await utils_1.deployTokenContract(ownerWallet, initialSupply);
            const blocks = await blockListener.waitForReceive(1, 5000);
            blocks.length.should.equal(1, 'Should have received 1 finalized block!');
            blocks[0].number.should.equal(0, 'Block 0 should have been finalized!');
            blocks[0].transactions.length.should.equal(0, 'There should be 0 transactions in block 0');
        }).timeout(timeout);
        it('finalizes multiple blocks after enough confirms', async () => {
            await blockProcessor.subscribe(provider, blockListener, false);
            tokenContract = await utils_1.deployTokenContract(ownerWallet, initialSupply);
            await tokenContract.transfer(ownerWallet.address, recipientWallet.address, sendAmount * 2);
            const blocks = await blockListener.waitForReceive(2, 5000);
            blocks.length.should.equal(2, 'Should have received 2 finalized block!');
            blocks[0].number.should.equal(0, 'Block 0 should have been finalized!');
            blocks[0].transactions.length.should.equal(0, 'There should be 0 transactions in block 0');
            blocks[1].number.should.equal(1, 'Block 1 should have been finalized!');
            blocks[1].transactions.length.should.equal(1, 'There should be 1 transactions in block 1');
            const deployToAddressEmpty = !blocks[1].transactions[0].to;
            deployToAddressEmpty.should.equal(true, 'The "to" address for the deploy tx should be null');
        }).timeout(timeout);
        describe('Syncing past blocks', () => {
            it('does not finalize past blocks if not final', async () => {
                await blockProcessor.subscribe(provider, blockListener, true);
                const blocks = await blockListener.waitForReceive(1, 5000);
                blocks.length.should.equal(0, 'Should not have finalized a block!');
            }).timeout(timeout);
            it('finalizes past block', async () => {
                tokenContract = await utils_1.deployTokenContract(ownerWallet, initialSupply);
                await blockProcessor.subscribe(provider, blockListener, true);
                const blocks = await blockListener.waitForReceive(1, 5000);
                blocks.length.should.equal(1, 'Should have received 1 finalized block!');
                blocks[0].number.should.equal(0, 'Block 0 should have been finalized!');
                blocks[0].transactions.length.should.equal(0, 'There should be 0 transactions in block 0');
            }).timeout(timeout);
            it('finalizes past and future blocks', async () => {
                tokenContract = await utils_1.deployTokenContract(ownerWallet, initialSupply);
                await blockProcessor.subscribe(provider, blockListener, true);
                await tokenContract.transfer(ownerWallet.address, recipientWallet.address, sendAmount * 2);
                const blocks = await blockListener.waitForReceive(2, 5000);
                blocks.length.should.equal(2, 'Should have received 2 finalized block!');
                blocks[0].number.should.equal(0, 'Block 0 should have been finalized!');
                blocks[0].transactions.length.should.equal(0, 'There should be 0 transactions in block 0');
                blocks[1].number.should.equal(1, 'Block 1 should have been finalized!');
                blocks[1].transactions.length.should.equal(1, 'There should be 1 transactions in block 1');
                const deployToAddressEmpty = !blocks[1].transactions[0].to;
                deployToAddressEmpty.should.equal(true, 'The "to" address for the deploy tx should be null');
            }).timeout(timeout);
            describe('Future earliest block', () => {
                it('does not finalize blocks before the earliest block', async () => {
                    tokenContract = await utils_1.deployTokenContract(ownerWallet, initialSupply);
                    blockProcessor = new app_1.EthereumBlockProcessor(db, 1, confirmsUntilFinal);
                    await blockProcessor.subscribe(provider, blockListener);
                    const blocks = await blockListener.waitForSyncToComplete();
                    blocks.length.should.equal(0);
                }).timeout(timeout);
                it('finalizes blocks after the earliest block', async () => {
                    tokenContract = await utils_1.deployTokenContract(ownerWallet, initialSupply);
                    await tokenContract.transfer(ownerWallet.address, recipientWallet.address, sendAmount * 2);
                    blockProcessor = new app_1.EthereumBlockProcessor(db, 1, confirmsUntilFinal);
                    await blockProcessor.subscribe(provider, blockListener);
                    const blocks = await blockListener.waitForSyncToComplete();
                    blocks.length.should.equal(1, `Incorrect number of blocks received!`);
                    blocks[0].number.should.equal(1, 'Block 1 should have been finalized!');
                    blocks[0].transactions.length.should.equal(1, 'There should be 1 transactions in block 1');
                    const deployToAddressEmpty = !blocks[0].transactions[0].to;
                    deployToAddressEmpty.should.equal(true, 'The "to" address for the deploy tx should be null');
                });
            });
        });
    });
});
//# sourceMappingURL=ethereum-block-processor.spec.js.map