"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_utils_1 = require("@eth-optimism/core-utils");
const ethers_1 = require("ethers");
const keystore_1 = require("../../../src/app/keystore");
const app_1 = require("../../../src/app");
const timeout = 15000;
describe('DefaultWallet', () => {
    let walletdb;
    let wallet;
    beforeEach(() => {
        walletdb = new keystore_1.DefaultWalletDB(app_1.newInMemoryDB());
        wallet = new keystore_1.DefaultWallet(walletdb);
    });
    describe('createAccount', () => {
        it('should correctly create an account', async () => {
            await wallet.createAccount('password').should.be.fulfilled;
        }).timeout(timeout);
    });
    describe('listAccounts', () => {
        it('should return an empty list if there are no accounts', async () => {
            const accounts = await wallet.listAccounts();
            accounts.should.deep.equal([]);
        });
        it('should return the account if an account exists', async () => {
            const account = await wallet.createAccount('password');
            const accounts = await wallet.listAccounts();
            accounts.should.deep.equal([account]);
        }).timeout(timeout);
        it('should return multiple accounts if more than one exists', async () => {
            const account1 = await wallet.createAccount('password');
            const account2 = await wallet.createAccount('password');
            const accounts = (await wallet.listAccounts()).sort();
            accounts.should.deep.equal([account1, account2].sort());
        }).timeout(timeout);
    });
    describe('unlockAccount', () => {
        it('should unlock an account if given the right password', async () => {
            const account = await wallet.createAccount('password');
            await wallet.unlockAccount(account, 'password').should.be.fulfilled;
        }).timeout(timeout);
        it('should throw if trying to unlock with the wrong password', async () => {
            const account = await wallet.createAccount('password');
            await wallet
                .unlockAccount(account, 'wrongpassword')
                .should.be.rejectedWith('Invalid account password.');
        }).timeout(timeout);
        it('should throw if the account does not exist', async () => {
            const account = core_utils_1.NULL_ADDRESS;
            await wallet
                .unlockAccount(account, 'password')
                .should.be.rejectedWith('Account does not exist.');
        });
    });
    describe('lockAccount', () => {
        it('should lock an account if unlocked', async () => {
            const account = await wallet.createAccount('password');
            await wallet.unlockAccount(account, 'password');
            await wallet.lockAccount(account).should.be.fulfilled;
        }).timeout(timeout);
        it('should lock an account even if not unlocked', async () => {
            const account = await wallet.createAccount('password');
            await wallet.lockAccount(account).should.be.fulfilled;
        }).timeout(timeout);
    });
    describe('sign', () => {
        it('should correctly sign some data', async () => {
            const account = await wallet.createAccount('password');
            await wallet.unlockAccount(account, 'password');
            const signature = await wallet.sign(account, 'hello world');
            ethers_1.ethers.utils.verifyMessage('hello world', signature).should.equal(account);
        }).timeout(timeout);
        it('should throw if the account is not unlocked', async () => {
            const account = await wallet.createAccount('password');
            await wallet
                .sign(account, 'hello world')
                .should.be.rejectedWith('Account is not unlocked.');
        }).timeout(timeout);
        it('should throw if the account does not exist', async () => {
            const account = '0x0000000000000000000000000000000000000000';
            await wallet
                .sign(account, 'hello world')
                .should.be.rejectedWith('Account does not exist.');
        });
    });
});
//# sourceMappingURL=wallet.spec.js.map